unit Ticker;

interface

{
  Example using TeeGrid with a TGridTicker component to automatically
  refresh grid cells with colors depending on cell value changes.
}

uses
  {Winapi.}Windows, {Winapi.}Messages, {System.}SysUtils, Variants, {System.}Classes,
  {Vcl.}Graphics, {Vcl.}Controls, {Vcl.}Forms, {Vcl.}Dialogs, {Vcl.}ExtCtrls,
  {Vcl.}ComCtrls, {Vcl.}StdCtrls, Grids, StrUtils,
  VCLTee.Control, VCLTee.Grid,

  Tee.GridData.Strings, Tee.Painter,
  Tee.Grid.Columns, Tee.Format, Tee.Renders,

  AvScreen;

type
  TCustomTeeGridAccess=class(TVCLTeeGrid);
  TTickerForm = class(TAvScreenForm)
    Section4: TTeeGrid;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CellEdited(const Sender: TObject; const AEditor: TControl;
      const AColumn: TColumn; const ARow: Integer; var ChangeData: Boolean;
      var NewData: string);
    procedure OnSelect(Sender: TObject);
  private
    { Private declarations }

    procedure FillColumns(PropInfo:String);
  public
    { Public declarations }
  end;

  function MvField(var BigStr:string; Delim:string; Dex:integer): string;

var
  TickerForm: TTickerForm;

implementation

{$R *.dfm}

uses
  System.UITypes, Tee.Grid, Tee.Grid.RowGroup;
var Cols: integer;
  Rows: integer = 7;
  Format4:TStringsData;
  VM: String = char(253);
  Data4:TStringsData;

procedure TTickerForm.FormCreate(Sender: TObject);
var PropInfo: String;
  Col, Row: integer;
begin
  isLoading := True;
  // Create grid data, for example "strings"
  Cols := Section4.Columns.Count;
  Data4:=TStringsData.Create(2,2);
  Format4:=TStringsData.Create(Cols,2);
  for Col := 0 to Cols-1 do begin
    Format4[Col,0] := VarToStr(Section4.Columns[Col].Width.Value);
    Format4[Col,1] := VarToStr(Section4.Columns[Col].TextAlign.Horizontal);
  end;

  Data4.Resize(Cols,Rows-1);
  Section4.Data:=Data4;

 //Section4.Data.Create;

  for Col := 0 to Cols-1 do begin
    Section4.Columns[Col].ParentFormat := False;
    Section4.Columns[Col].Width.Value := StrToInt(Format4[Col,0]);
    //ShowMessage(Just4[Col,0]);
    if (Format4[Col,1] = '1') or (Format4[Col,1] = '2') then
      begin
        Section4.Columns[Col].TextAlignment := TColumnTextAlign.Custom;
        if (Format4[Col,1] = '1') then
          Section4.Columns[Col].TextAlign.Horizontal := THorizontalAlign.Center
        else
          Section4.Columns[Col].TextAlign.Horizontal := THorizontalAlign.Right;
      end;
  end;

  //Blocked Column Format;
  Section4.Columns[4].Format.Brush.Visible := True;
  Section4.Columns[4].Format.Brush.Color := 2158272672;
//  Section4.Columns[4].ParentFormat := False;
  Section4.Columns[4].ReadOnly := True;
  Section4.Columns[4].Selectable := False;

  //Alternate Row Format
  Section4.Rows.Format.Brush.Visible := True;
  Section4.Rows.Alternate.Brush.Color := clSkyBlue;
  Section4.Rows.Alternate.Visible := True;

  // Fill data with test values
  //ShowMessage('Loading column 0');
  PropInfo := 'Grid' + VM + 'Section4' + VM + '"Store is the and;jvbna;FVB;jbvn;ijbv ;aJFEV;jfv ;i ","1"," "," "," ","10"," "' + VM + '0';
  FillColumns(PropInfo);
  //ShowMessage('Loading column 1');
  PropInfo := 'Grid' + VM + 'Section4' + VM + '"Term","95","1,1","1""","9.345"," ","39"' + VM + '1';
  FillColumns(PropInfo);
  //ShowMessage('Loading column 2');
  PropInfo := 'Grid' + VM + 'Section4' + VM + '"Status","HCDE","HCWA","HOFP","INPR","HFOS","INPR"' + VM + '2';
  FillColumns(PropInfo);
  //ShowMessage('Loading column 3');
  PropInfo := 'Grid' + VM + 'Section4' + VM + '"Count","3","5","4","83","1","38"' + VM + '3';
  FillColumns(PropInfo);
  //ShowMessage('Loading column 4');
  PropInfo := 'Grid' + VM + 'Section4' + VM + '"Pickable","0","0","0","16","0","36"' + VM + '4';
  FillColumns(PropInfo);

  Section4.Selected.Change(Section4.Columns[2],2);
//  TCustomTeeGridAccess(Section4.Grid).StartEditor;
end;

procedure TTickerForm.FormDestroy(Sender: TObject);
begin
  //Ticker.Free; // avoid memory leak
end;

procedure TTickerForm.FormShow(Sender: TObject);
begin
  // Start updating values
  isLoading := False;
end;

procedure TTickerForm.OnSelect(Sender: TObject);
begin
  inherited;
  //do nothing here
end;

procedure TTickerForm.CellEdited(const Sender: TObject;
  const AEditor: TControl; const AColumn: TColumn; const ARow: Integer;
  var ChangeData: Boolean; var NewData: string);
begin
  inherited;
  ShowMessage('edited ticker form');
end;

//Initialize data headers
procedure TTickerForm.FillColumns(PropInfo:string);
var AvObjectName: string;
    AvObjectType: string;
    AvPropertyName: string;
    AvPropertyType: string;
    AvPropertyValue: string;
    ColData: TStringList;
    ColumnNumber: integer;
    GridObject: TTeeGrid;
    NumCols: integer;
    Row: integer;
    VM: string;
begin
  VM := char(253);
  AvObjectName := MvField(PropInfo,VM,1);
  AvObjectType := MidStr(AvObjectName,1,2);
  AvPropertyName := MvField(PropInfo,VM,2);
  AvPropertyValue := MvField(PropInfo,VM,3);
  AvPropertyType := MvField(PropInfo,VM,4);
  ColData := nil;

  if AvObjectName = 'Grid' then begin
      GridObject := TTeeGrid(FindComponent(AvPropertyName));
      ColumnNumber := StrToInt(AvPropertyType);
      NumCols := GridObject.Columns.Count;
      if NumCols > ColumnNumber then begin
        try
          ColData := TStringList.Create;
          ColData.CommaText := AvPropertyValue;
          begin
            for Row := 0 to ColData.Count-1 do
              if Row = 0 then
                Section4.Columns[ColumnNumber].Header.Text := ColData[Row]
              else
                Section4.Data.SetValue(Section4.Columns[ColumnNumber],Row-1,ColData[Row]);
          end;
        finally
          ColData.Free;
        end;
      end;
  end;
end;

function MvField(var BigStr:string; Delim:string; Dex:integer): string;
var Col1,I,Lenn:integer;
    CutStr: string;
begin
  Result := '';
  CutStr := BigStr;
  for I := 1 to Dex do begin
    Col1 := Pos(Delim,CutStr);
    if Col1 = 0 then
      Col1 := Length(CutStr)+1;
    Lenn := Col1-1;
    if I<Dex then
      Delete(CutStr,1,Col1)
    else
      Result := MidStr(CutStr,1,Lenn);
  end;
end;

end.
